# coding: utf-8

"""
Gluish - few utils for luigi.
"""

__version__ = '0.2.0'

from gluish.common import *
from gluish.format import *
from gluish.intervals import *
from gluish.parameter import *
from gluish.task import *
from gluish.utils import *



#!/usr/bin/env python
# coding: utf-8

import luigi
import os

__all__ = ['Executable']

def which(program):
    """
    Return `None` if no executable can be found.
    """
    def is_exe(fpath):
        """ Is `fpath` executable? ` """
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    fpath, fname = os.path.split(program)
    if fpath:
        if is_exe(program):
            return program
    else:
        for path in os.environ["PATH"].split(os.pathsep):
            path = path.strip('"')
            exe_file = os.path.join(path, program)
            if is_exe(exe_file):
                return exe_file

    return None

class Executable(luigi.Task):
    """
    Checks, whether an external executable is available. This task will consider
    itself complete, only if the executable `name` is found in $PATH.
    """
    name = luigi.Parameter()
    message = luigi.Parameter(default="")

    def run(self):
        """ Only run if, task is not complete. """
        raise RuntimeError('external program required: %s (%s)' % (self.name, self.message))

    def complete(self):
        return which(self.name) is not None



# coding: utf-8
# pylint: disable=C0301

"""
A few utilities.
"""

from dateutil import relativedelta
import logging
import random
import re
import string
import subprocess
import tempfile

__all__ = [
    'date_range',
    'random_string',
    'shellout',
]

logger = logging.getLogger('gluish')

def random_string(length=16):
    """
    Return a random string (upper and lowercase letters) of length `length`,
    defaults to 16.
    """
    return ''.join(random.choice(string.ascii_letters) for _ in range(length))

def date_range(start_date, end_date, increment, period):
    """
    Generate `date` objects between `start_date` and `end_date` in `increment`
    `period` intervals.
    """
    next = start_date
    delta = relativedelta.relativedelta(**{period:increment})
    while next <= end_date:
        yield next
        next += delta

def shellout(template, **kwargs):
    """
    Takes a shell command template and executes it. The template must use
    the new (2.6+) format mini language. `kwargs` must contain any defined
    placeholder, only `output` is optional.
    Raises RuntimeError on nonzero exit codes.

    Simple template:

        wc -l < {input} > {output}

    Quoted curly braces:

        ps ax|awk '{{print $1}}' > {output}

    Usage with luigi:

        ...
        tmp = shellout('wc -l < {input} > {output}', input=self.input().path)
        luigi.File(tmp).move(self.output().path)
        ....

    """
    preserve_whitespace = kwargs.get('preserve_whitespace', False)
    if not 'output' in kwargs:
        kwargs.update({'output': tempfile.mkstemp(prefix='gluish-')[1]})
    ignoremap = kwargs.get('ignoremap', {})
    encoding = kwargs.get('encoding', None)
    if encoding:
        command = template.decode(encoding).format(**kwargs)
    else:
        command = template.format(**kwargs)
    if not preserve_whitespace:
        command = re.sub('[ \t\n]+', ' ', command)
    logger.debug(command)
    code = subprocess.call([command], shell=True)
    if not code == 0:
        if code in ignoremap:
            logger.info("Ignoring error via ignoremap: %s" % ignoremap.get(code))
        else:
            logger.error('%s: %s' % (command, code))
            error = RuntimeError('%s exitcode: %s' % (command, code))
            error.code = code
            raise error
    return kwargs.get('output')



# coding: utf-8

"""
Some glue around luigi.

Provides a base class, that autogenerates its output filenames based on some
tag, classname and parameters.

Additionally, provide some smaller utilities, like a TSV format.
"""

from setuptools import setup

install_requires = [
    'luigi>=1.0.20',
    'python-dateutil==2.2',
    'pytz==2014.4',
    'six==1.9.0',
]

setup(name='gluish',
      version='0.2.0',
      description='Luigi helper.',
      url='https://github.com/miku/gluish',
      author='Martin Czygan',
      author_email='martin.czygan@gmail.com',
      packages=[
        'gluish',
      ],
      package_dir={'gluish': 'gluish'},
      install_requires=install_requires,
)



# coding: utf-8
# pylint: disable=F0401,C0103,R0921,E1101,W0232,R0201,R0903,E1002

"""
Custom luigi parameters.
"""

import luigi

__all__ = ['ClosestDateParameter']

class ClosestDateParameter(luigi.DateParameter):
    """
    A marker parameter to replace date parameter value with whatever
    self.closest() returns. Use in conjunction with `gluish.task.BaseTask`.
    """
    use_closest_date = True
